/*
Copyright (c) AppDynamics, Inc., and its affiliates
2015
All Rights Reserved
 */
'use strict';

module.exports.getHttpStatusCodeConfig = getHttpStatusCodeConfig;
module.exports.generateError = generateError;
module.exports.getHttpExitCallError = getHttpExitCallError;

function getHttpStatusCodeConfig(statusCode, statusCodesConfig) {
  if (!statusCodesConfig) {
    return undefined;
  }

  for (var i = 0, length = statusCodesConfig.length; i < length; ++i) {
    var statusCodeConfig = statusCodesConfig[i];
    if (statusCode >= statusCodeConfig.lowerBound &&
      statusCode <= statusCodeConfig.upperBound) {
      return statusCodeConfig;
    }
  }
  return undefined;
}

function generateError(exception, statusCode, statusCodesConfig) {
  var errorObject;
  if (exception) {
    if (exception instanceof Error) {
      errorObject = {
        name: "JS " + exception.name,
        message: exception.message,
        stack: exception.stack
      };
    }
  } else {
    var statusCodeConfig = getHttpStatusCodeConfig(statusCode, statusCodesConfig);
    if (statusCodeConfig) {
      if (statusCodeConfig.enabled) {
        errorObject= { message: statusCodeConfig.description.toString(), name: "Error" };
      }
    }
    else if ((statusCode < 200) || (statusCode >= 400)) {
      var error, errorMsg;
      if (statusCode >= 400 && statusCode <= 505) {
        error = "HTTP response code " + statusCode;
      } else {
        error = "HTTP response code other";
      }
      errorMsg = "HTTP response code " + statusCode;
      errorObject = { message: errorMsg, name: error };
    }
  }
  return errorObject;
}

function getHttpExitCallError(statusCode, stack, locals) {
  var error = {message: "Error code " + statusCode + " for URL " + locals.opts.hostname + ":" + locals.opts.port + locals.opts.path,
    name: "Error",
    stack: stack};
  return error;
}
